/* Temperature.cpp defines the non-trivial Temperature operations.
 *  ...
 ****************************************************************/

#include "Temperature.h"                  // class Temperature
#include <cctype>                        // islower(), toupper()
#include <cstdlib>                       // exit()
//#include <iomanip>
using namespace std;

// -------- Explicit-value constructor --------------------------

Temperature::Temperature(double initialDegrees, char initialScale)
{
  if ( isValid(initialDegrees, initialScale) )
  {
    if (islower(initialScale))             // if scale is lowercase
      initialScale = toupper(initialScale); // convert it to uppercase

    myDegrees = initialDegrees;      //  proceed with
    myScale = initialScale;          //  initialization
  }                                      // otherwise, error msg
 else
 {
     cerr << "\n*** Temperature constructor received invalid params "
          << initialDegrees << ' ' << initialScale << endl;
     exit(1);
  }
}

// -------- Utility method -----------------------------------

bool Temperature::isValid(double degrees, char scale)
{

   switch (scale)
   {
      case 'F': case 'f':                 // Fahrenheit
         return degrees >= MIN_FAHRENHEIT;
      case 'C': case 'c':                 // Celsius
         return degrees >= MIN_CELSIUS;
      case 'K': case 'k':                 // Kelvin
         return degrees >= MIN_KELVIN;
      default:                            // otherwise, invalid
         return false;
   }
}

// -------- Temperature Input -------------------------------------

void Temperature::read(istream& in)
{
  double inDegrees;                  // temporary variables to
  char inScale;                      //  store the input values
  in >> inDegrees >> inScale;        // read values from in

  if ( isValid(inDegrees, inScale) ) // if they're valid
  {
    if ( islower(inScale) )          // if scale is lower case
      inScale = toupper(inScale);    //   convert it to upper case

    myScale = inScale;               //   assign input values
    myDegrees = inDegrees;           //    to instance variables
  }
  else                               // otherwise
    in.setstate(ios::failbit);       //   set fail bit in stream
}                                    //    and return


// -------- Fahrenheit conversion method -----------------



Temperature Temperature::inFahrenheit() const

{

   switch (myScale)

   {

      case 'F': 

         return Temperature(myDegrees, 'F');

      case 'C':

         return Temperature(myDegrees * 1.8 + 32.0, 'F');

      case 'K':

         return Temperature((myDegrees - 273.15) * 1.8 + 32.0, 'F');

	  default:

		 return Temperature(0.0, 'F');

   }

}



// -------- Celsius conversion method -----------------



Temperature Temperature::inCelsius() const

{

   switch (myScale)

   {

      case 'F': 

         return Temperature((myDegrees - 32.0) / 1.8, 'C');

      case 'C':

         return Temperature(myDegrees, 'C');

      case 'K':

         return Temperature(myDegrees - 273.15, 'C');

	  default:

		 return Temperature(0.0, 'C');

   }

}



// -------- Kelvin conversion method -----------------



Temperature Temperature::inKelvin() const

{

   switch (myScale)

   {

      case 'F': 

         return Temperature((myDegrees - 32.0) / 1.8 + 273.15, 'K');

      case 'C':

         return Temperature(myDegrees + 273.15, 'K');

      case 'K':

         return Temperature(myDegrees, 'K');

	  default:

		 return Temperature(0.0, 'K');

   }

}

// --------- Relational comparison utility ---------------------

int Temperature::compare(const Temperature& rightOperand) const
{
   Temperature selfInKelvin = inKelvin(),
               rightOperandInKelvin = rightOperand.inKelvin();
               
   return selfInKelvin.getDegrees() - rightOperandInKelvin.getDegrees();
}

